/*  _   _  ___  __  __ _____   _____  _    ____   */
/* | | | |/ _ \|  \/  | ____| |_   _|/ \  | __ )  */
/* | |_| | | | | |\/| |  _|     | | / _ \ |  _ \  */
/* |  _  | |_| | |  | | |___    | |/ ___ \| |_) | */
/* |_| |_|\___/|_|  |_|_____|   |_/_/   \_\____/  */

#--content-tab-HomeTab{color: green;}

#InputProjectName{

    layout: horizontal;
    height: 3;
    height: auto;

    #Project_name{
        width: auto;
        margin: 1;
    }

    #Input_Project_name{
        height: 3;
        border: wide $background;
        min-width: 20;
        width: auto;
    }

    #Project_location{
        margin: 1;
        width: auto;
        opacity: 60%;
    }
}

#Download_widget{
    width: auto;
    background: $boost;
    height: 12;
    .running #Download_loading{display: block;}
    #download_title{
        background: $panel-darken-2;
        width: 100%;
        margin: 0 0 1 0;
        padding: 1;
        height: 3;
        content-align-vertical: middle;
    }
    #Input_url{
        margin: 0 0 0 1;
        width: 70%;
    }
    #Input_outputfile_name{
        margin: 0 0 0 0;
        width: 30%;
    }
    #Download_options{
        margin: 1;
        Label{
            margin: 1;
            height: 3;
        }
        Button{
            margin: 0 1 0 0;
        }
    }
    LoadingIndicator{
        width: 16;
        height: 3;
        /* dock: right; */
        display: none;
    }
}

Container #FastQC_widget, #MultiQC_widget, #bwa_widget{
    width: auto;
    background: $boost;
    height: 13;
    margin: 1 0 0 0;
    .running #FastQC_Loading{display: block;}
    .running #MultiQC_Loading{display: block;}
    .running #bwa_Loading{display: block;}
    #FastQC_title, #MultiQC_title, #bwa_title{
        background: $panel-darken-2;
        width: 100%;
        margin: 0 0 1 0;
        padding: 1;
        height: 3;
    }
    Input{
        margin: 0 0 0 1;
        width: 99%;
    }
    Button{
        margin: 1;
    }

    LoadingIndicator{
        width: 16;
        height: 3;
        dock: right;
        display: none;
        margin: 1;
    }
}

#FastQC_widget{
    Label{
        margin: 1 1 1 2;
        width: 70%;
    }
    #FastQC_input_input,#FastQC_output_input{
        width: 49%;
    }
}

#MultiQC_widget{
    Label{
        margin: 1 1 1 2;
        width: 70%;
    }
    #MultiQC_input_input,#MultiQC_output_input{
        width: 49%;
    }
}

#bwa_widget{
    height: 20;
    Input{
        /* margin: 0 0 1 1; */
        width: 99%;
    }
    Label{
        margin: 1 1 1 2;
        width: 60%;
    }
    #bwa_reads_input_1, #bwa_reads_input_2{
        margin: 0 0 0 1;
        width: 49%;
    }
    #bwa_ref_input{
        margin: 0 0 0 1;
        width: 75%;
    }
    #bwa_threads_input{
        margin: 0 0 0 1;
        width: 24%;
    }
    #bwa_output_input{
        margin: 0 0 0 1;
        width: 98%;
    }

}

/*  ____    _    __  __ _____ ___   ___  _     ____    _____  _    ____   */
/* / ___|  / \  |  \/  |_   _/ _ \ / _ \| |   / ___|  |_   _|/ \  | __ )  */
/* \___ \ / _ \ | |\/| | | || | | | | | | |   \___ \    | | / _ \ |  _ \  */
/*  ___) / ___ \| |  | | | || |_| | |_| | |___ ___) |   | |/ ___ \| |_) | */
/* |____/_/   \_\_|  |_| |_| \___/ \___/|_____|____/    |_/_/   \_\____/  */

#sam_view_container, #sam_sort_container, #sam_index_container{
    Label{
        height: 3;
        padding: 0 1 0 1;
        background: $panel-darken-2;
        width: 100%;
        content-align-vertical: middle;
    }
    .description{
        background: $boost;
    }
    background: $boost;
    margin: 0 0 1 0;
    height: 11;
}

#sam_view_horizontal{
    height: auto;
    margin: 1;
    Input{
        width: 29%;
    }
}

#sam_sort_horizontal{
    height: auto;
    margin: 1;
    Input{
        width: 45%;
    }
}

#sam_index_horizontal{
    height: auto;
    margin: 1;
    Input{
        width: 90%;
    }
}

/*
 ____   ____ _____ _____ ___   ___  _     ____    _____  _    ____
| __ ) / ___|  ___|_   _/ _ \ / _ \| |   / ___|  |_   _|/ \  | __ )
|  _ \| |   | |_    | || | | | | | | |   \___ \    | | / _ \ |  _ \
| |_) | |___|  _|   | || |_| | |_| | |___ ___) |   | |/ ___ \| |_) |
|____/ \____|_|     |_| \___/ \___/|_____|____/    |_/_/   \_\____/
*/
#bcf_mpileup_container, #bcf_call_container, #bcf_filter_container, #bcf_norm_container, #bcf_stats_container{
    Label{
        height: 3;
        padding: 0 1 0 1;
        background: $panel-darken-2;
        width: 100%;
        content-align-vertical: middle;
    }
    .description{
        background: $boost;
    }
    background: $boost;
    margin: 0 0 1 0;
    height: 11;
}

#bcf_mpileup_horizontal, #bcf_filter_horizontal{
    height: auto;
    margin: 1;
    Input{
        width: 30%;
    }
}

#bcf_call_horizontal, #bcf_norm_horizontal, #bcf_stats_horizontal{
    height: auto;
    margin: 1;
    Input{
        width: 45%;
    }
}


/*
 ____ ___ ____  _____ _     ___ _   _ _____   _____  _    ____
|  _ \_ _|  _ \| ____| |   |_ _| \ | | ____| |_   _|/ \  | __ )
| |_) | || |_) |  _| | |    | ||  \| |  _|     | | / _ \ |  _ \
|  __/| ||  __/| |___| |___ | || |\  | |___    | |/ ___ \| |_) |
|_|  |___|_|   |_____|_____|___|_| \_|_____|   |_/_/   \_\____/
*/

#Pipeline_container{
    Label{
        margin: 1 1 1 2;
        background: $boost;
        height: 3;
        width: 99%;
        content-align-vertical: middle;
        padding: 0 1 0 1;
    }
    Input{
        margin: 1;
    }
    Button{
        margin: 1 1 1 2;
    }
}

/*
_   _ _____ _     ____    _____  _    ____
| | | | ____| |   |  _ \  |_   _|/ \  | __ )
| |_| |  _| | |   | |_) |   | | / _ \ |  _ \
|  _  | |___| |___|  __/    | |/ ___ \| |_) |
|_| |_|_____|_____|_|       |_/_/   \_\____/
*/

#--content-tab-HelpTab{color: #1691CE;}
